@@ -61,7 +61,7 @@ class ExpressionEdit : public QPlainTextEdit {
 		QStandardItemModel *sourceModel;
 		QTableView *completionView;
 		QMenu *cmenu;
-		QAction *undoAction, *redoAction, *cutAction, *copyAction, *pasteAction, *deleteAction, *selectAllAction, *clearAction, *statusOffAction, *statusDelayAction, *statusNoDelayAction;
+		QAction *undoAction, *redoAction, *cutAction, *copyAction, *pasteAction, *deleteAction, *selectAllAction, *clearAction, *statusOffAction, *statusDelayAction, *statusNoDelayAction, *clearHistoryAction;
 		QTimer *completionTimer, *toolTipTimer;
 		ExpressionTipLabel *tipLabel;
 		QWidget *tb;
@@ -95,6 +95,8 @@ class ExpressionEdit : public QPlainTextEdit {
 
 		void keyReleaseEvent(QKeyEvent*) override;
 		void contextMenuEvent(QContextMenuEvent *e) override;
+		void insertFromMimeData(const QMimeData*) override;
+		bool expressionInQuotes();
 
 	public:
 
@@ -147,6 +149,7 @@ class ExpressionEdit : public QPlainTextEdit {
 		void insertMatrix();
 		void completeOrActivateFirst();
 		bool complete(MathStructure* = NULL, const QPoint& = QPoint(), bool = false);
+		void clearHistory();
 
 	signals:
 
